package org.KeyWrapping;

import javacard.framework.ISO7816;
import javacard.framework.ISOException;
import javacard.framework.JCSystem;
import javacard.framework.Util;
import javacard.security.CryptoException;
import javacard.security.Key;
import javacardx.crypto.Cipher;

public class AES_CTR extends Cipher {
    public static final byte AES_BLOCK_SIZE = 16;
    public static final byte IV_SIZE = 16;
    private Cipher aesEngine = null;

    private byte[] iv = null;
    private byte[] encryptedIV = null;


    public AES_CTR() {
        aesEngine = Cipher.getInstance(Cipher.ALG_AES_BLOCK_128_ECB_NOPAD, false);
        iv = JCSystem.makeTransientByteArray(IV_SIZE, JCSystem.CLEAR_ON_DESELECT);
        encryptedIV = JCSystem.makeTransientByteArray(IV_SIZE, JCSystem.CLEAR_ON_DESELECT);
    }

    @Override
    public void init(Key key, byte mode) throws CryptoException {
        aesEngine.init(key, Cipher.MODE_ENCRYPT);
    }

    @Override
    public void init(Key key, byte mode, byte[] nonce, short nOff, short nLen) throws CryptoException {
        if(nLen != AES_BLOCK_SIZE) {
            throw new CryptoException(CryptoException.ILLEGAL_VALUE);
        }
        Util.arrayCopy(nonce, nOff, this.iv, (short) 0, nLen);
        this.init(key, Cipher.MODE_ENCRYPT);
    }

    @Override
    public short doFinal(byte[] inBuff, short inOffset, short inLength, byte[] outBuff, short outOffset) throws CryptoException {

        short blockStartOff = outOffset;
        final short blockCount = (short) (inLength / AES_BLOCK_SIZE);
        if(inBuff != outBuff)
            Util.arrayCopyNonAtomic(inBuff, inOffset, outBuff, outOffset, inLength);

        // Main cipher loop
        for(short blockIndex = 0; blockIndex < blockCount; blockIndex++) {
            processCTRBlock(AES_BLOCK_SIZE, outBuff, blockStartOff);

            incrementByteArray(iv);
            blockStartOff += AES_BLOCK_SIZE;
        }
        // Handle last block
        short lastBlockLen = (short) (inLength - (blockCount * AES_BLOCK_SIZE));
        if(lastBlockLen > 0)
            processCTRBlock(lastBlockLen, outBuff, blockStartOff);

        Util.arrayFillNonAtomic(iv, (short) 0, IV_SIZE, (byte) 0);
        return (short) (blockCount * AES_BLOCK_SIZE + lastBlockLen);
    }

    @Override
    public short update(byte[] inBuff, short inOffset, short inLength, byte[] outBuff, short outOffset) throws CryptoException {
        ISOException.throwIt(ISO7816.SW_FUNC_NOT_SUPPORTED);
        return 0;
    }

    private void processCTRBlock(short cLen, byte[] outArray, short outOff) {
        aesEngine.doFinal(iv, (short) 0, IV_SIZE, encryptedIV, (short) 0);
        short outIndex = outOff;
        for(short ivIndex = 0; ivIndex < cLen; ivIndex++) {
            outArray[outIndex++] ^= encryptedIV[ivIndex];
        }
    }

    public static void incrementByteArray(byte[] bArray) {
        short carry = 1;
        for(short i = (short) (bArray.length - 1); i >= 0 && carry == 1; i--) {
            bArray[i] += carry;
            carry = (short) (bArray[i] == 0 ? 1 : 0);
        }
    }


    @Override
    public byte getAlgorithm() {
        return 0;
    }
}
