package org.KeyWrapping;

import javacard.framework.ISO7816;
import javacard.framework.ISOException;
import javacard.framework.JCSystem;
import javacard.framework.Util;
import javacard.security.MessageDigest;

public class HMAC {
    public static final short SHA256_BLOCK_SIZE = 64;
    public static final short SHA256_OUTPUT_SIZE = 32;
    private static final byte IPAD = 0x36;
    private static final byte OPAD = 0x5C;
    private MessageDigest mdEngine = null;
    private byte[] keyBuffer = null;

    public HMAC() {
        mdEngine = MessageDigest.getInstance(MessageDigest.ALG_SHA_256, false);
        keyBuffer = JCSystem.makeTransientByteArray(SHA256_BLOCK_SIZE, JCSystem.CLEAR_ON_DESELECT);
    }

    public void init(HMACKey key, byte mode) {
        if(mode != MessageDigest.ALG_SHA_256) {
            ISOException.throwIt(ISO7816.SW_FUNC_NOT_SUPPORTED);
        }
        short keyLength = (short) (key.getSize() / 8);
        if(keyLength > SHA256_BLOCK_SIZE) {
            byte[] tempKey = new byte[keyLength];
            mdEngine.doFinal(tempKey, (short) 0, keyLength, keyBuffer, SHA256_OUTPUT_SIZE);
            keyLength = SHA256_OUTPUT_SIZE;
        } else {
            key.getKey(keyBuffer, (short) 0);
        }

        createPaddedKey(IPAD, keyBuffer, keyBuffer, keyLength);
        // Already start the engine
        mdEngine.update(keyBuffer, (short) 0, SHA256_BLOCK_SIZE);

        key.getKey(keyBuffer, (short) 0);
        createPaddedKey(OPAD, keyBuffer, keyBuffer, keyLength);
    }

    public short doFinal(byte[] inBuff, short inOffset, short inLength, byte[] outBuff, short outOffset) {
        mdEngine.doFinal(inBuff, inOffset, inLength, outBuff, outOffset);

        mdEngine.update(keyBuffer, (short) 0, SHA256_BLOCK_SIZE);
        mdEngine.doFinal(outBuff, outOffset, SHA256_OUTPUT_SIZE, outBuff, outOffset);

        return SHA256_OUTPUT_SIZE;
    }

    public void update(byte[] inBuff, short inOffset, short inLength) {
        mdEngine.update(inBuff, inOffset, inLength);
    }

    private void createPaddedKey(byte pattern, byte[] byteKey, byte[] hmacKey, short keyLength) {
        for(short i = 0; i < keyLength; i++) {
            hmacKey[i] = (byte) (pattern ^ byteKey[i]);
        }
        Util.arrayFillNonAtomic(hmacKey, keyLength, (short) (SHA256_BLOCK_SIZE - keyLength), pattern);
    }
}
